@@ -1,6 +1,8 @@
 #include "gd.h"
 #include "gdhelpers.h"
 #include <stdlib.h>
+#include <string.h>
+#include <limits.h>
 
 /* TBB: gd_strtok_r is not portable; provide an implementation */
 
@@ -10,7 +12,6 @@ char *
 gd_strtok_r (char *s, char *sep, char **state)
 {
   char separators[256];
-  char *start;
   char *result = 0;
   memset (separators, 0, sizeof (separators));
   while (*sep)
@@ -23,7 +24,6 @@ gd_strtok_r (char *s, char *sep, char **state)
       /* Pick up where we left off */
       s = *state;
     }
-  start = s;
   /* 1. EOS */
   if (!(*s))
     {
@@ -93,3 +93,18 @@ gdFree (void *ptr)
 {
   free (ptr);
 }
+
+int overflow2(int a, int b)
+{
+	if(a < 0 || b < 0) {
+		fprintf(stderr, "gd warning: one parameter to a memory allocation multiplication is negative, failing operation gracefully\n");
+		return 1;
+	}
+	if(b == 0)
+		return 0;
+	if(a > INT_MAX / b) {
+		fprintf(stderr, "gd warning: product of memory allocation multiplication would exceed INT_MAX, failing operation gracefully\n");
+		return 1;
+	}
+	return 0;
+}
